#
# coding=utf-8
"""test_isbn13 - Test Isbn13 class"""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from unittest import TestCase

from expecter import expect

from pyisbn import Isbn13


class TestIsbn13(TestCase):
    def test_calculate_checksum(self):
        expect(Isbn13('978-052-187-1723').calculate_checksum()) == '3'

    def test_convert(self):
        expect(Isbn13('9780071148160').convert()) == '0071148167'



#
# coding=utf-8
"""test_functions - Test function interface"""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import unicodedata

from expecter import expect
from nose2.tools import params


from pyisbn import (IsbnError, _isbn_cleanse, calculate_checksum, convert,
                    validate)
from tests.test_data import TEST_BOOKS


@params(*TEST_BOOKS.values())
def test__isbn_cleanse_sbn(isbn):
    if isbn.startswith('0'):
        expect(_isbn_cleanse(isbn[1:])) == isbn.replace('-', '')
        expect(_isbn_cleanse(isbn[1:-1], False)) == isbn.replace('-', '')[:-1]


@params(*TEST_BOOKS.values())
def test__isbn_cleanse_isbn(isbn):
    expect(_isbn_cleanse(isbn)) == isbn.replace('-', '')
    expect(_isbn_cleanse(isbn[:-1], False)) == isbn.replace('-', '')[:-1]


# See tests.test_regressions.test_issue_7_unistr
@params(
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    unicodedata.lookup('EM DASH').join(['978', '0', '385', '08695', '0']),
    unicodedata.lookup('HORIZONTAL BAR').join(['978', '0199564095']),
)
def test__isbn_cleanse_unicode_dash(isbn):
    expect(_isbn_cleanse(isbn)) == "".join(filter(lambda s: s.isdigit(), isbn))


@params(
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    '978-0-385-08695-0',
)
def test__isbn_cleanse_reflect_type(isbn):
    expect(type(_isbn_cleanse(isbn))) == type(isbn)


def test__isbn_cleanse_invalid_type():
    with expect.raises(TypeError, "ISBN must be a string, received 2"):
        _isbn_cleanse(2)


@params(
    (True, 'ISBN must be either 10 or 13 characters long'),
    (False, 'ISBN must be either 9 or 12 characters long without checksum'),
)
def test__isbn_cleanse_invalid_length(checksum, message):
    with expect.raises(IsbnError, message):
        _isbn_cleanse('0-123', checksum=checksum)


@params(
    ('0-x4343', 'non-digit parts'),
    ('012345678-b', 'non-digit or X checksum'),
    ('012345678901b', 'non-digit checksum'),
    ('xxxxxxxxxxxx1', 'non-digit parts'),
)
def test__isbn_cleanse_invalid(isbn, message):
    with expect.raises(IsbnError, message):
        _isbn_cleanse(isbn)


@params(*TEST_BOOKS.values())
def test_calculate_checksum(isbn):
    expect(calculate_checksum(isbn[:-1])) == isbn[-1]


@params(*TEST_BOOKS.values())
def test_convert(isbn):
    expect(convert(convert(isbn))) == isbn.replace('-', '')


def test_convert_invalid():
    with expect.raises(IsbnError,
                       'Only ISBN-13s with 978 Bookland code can be converted '
                       'to ISBN-10.'):
        convert('0000000000000')


@params(*TEST_BOOKS.values())
def test_validate(isbn):
    expect(validate(isbn)) == True


@params(
    ('1-234-56789-0', ),
    ('2-345-6789-1', ),
    ('3-456-7890-X', )
)
def test_validate_invalid(isbn):
    expect(validate(isbn)) == False



#
# coding=utf-8
"""pyisbn - A module for working with 10- and 13-digit ISBNs."""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from pyisbn import _version


__version__ = _version.dotted
__date__ = _version.date
__author__ = 'James Rowe <jnrowe@gmail.com>'
__copyright__ = 'Copyright © 2007-2016  James Rowe'
__license__ = 'GNU General Public License Version 3'
__credits__ = ''
__history__ = 'See git repository'

try:
    from email.utils import parseaddr
except ImportError:  # Python 2.4
    from email.Utils import parseaddr  # NOQA

__doc__ += """.

This module supports the calculation of ISBN checksums with
``calculate_checksum()``, the conversion between ISBN-10 and ISBN-13 with
``convert()`` and the validation of ISBNs with ``validate()``.

All the ISBNs must be passed in as ``str`` types, even if it would seem
reasonable to accept some ``int`` forms.  The reason behind this is English
speaking countries use ``0`` for their group identifier, and Python would treat
ISBNs beginning with ``0`` as octal representations producing incorrect results.
While it may be feasible to allow some cases as non-``str`` types the complexity
in design and usage isn't worth the minimal benefit.

The functions in this module also support 9-digit SBNs for people with older
books in their collection.

:version: %s
:author: `%s <mailto:%s>`__
:copyright: %s
:status: Stable
:license: %s
""" % ((__version__, ) + parseaddr(__author__) + (__copyright__, __license__))

import unicodedata

from sys import version_info

PY2 = version_info[0] == 2

if PY2:
    string_types = (str, unicode)
else:
    string_types = (str, )
    unicode = str

#: Dash types to accept, and scrub, in ISBN inputs
DASHES = [unicodedata.lookup(s) for s in ('HYPHEN-MINUS', 'EN DASH', 'EM DASH',
                                          'HORIZONTAL BAR')]

#: Site to URL mappings, broken out for easier extending at runtime
URL_MAP = {
    'amazon': (
        ('http://www.amazon.%(tld)s/s'
         '?search-alias=stripbooks&field-isbn=%(isbn)s'),
        {
            'de': None,
            'fr': None,
            'jp': None,
            'uk': 'co.uk',
            'us': 'com',
        }),
    'copac': 'http://copac.ac.uk/search?isn=%(isbn)s',
    'google': 'http://books.google.com/books?vid=isbn:%(isbn)s',
    'isbndb': 'http://isbndb.com/search/all?query=%(isbn)s',
    'worldcat': 'http://worldcat.org/isbn/%(isbn)s',
    'waterstones': ('http://www.waterstones.com/waterstonesweb/'
                    'advancedSearch.do?buttonClicked=2&isbn=%(isbn)s'),
    'whsmith': ('http://www.whsmith.co.uk/CatalogAndSearch/'
                'SearchWithinCategory.aspx?as_ISBN=%(isbn)s')
}


class CountryError(ValueError):

    """Unknown country value."""


class IsbnError(ValueError):

    """Invalid ISBN string."""


class SiteError(ValueError):

    """Unknown site value."""


class Isbn(object):

    """Class for representing ISBN objects."""

    __slots__ = ('__weakref__', '_isbn', 'isbn')

    def __init__(self, isbn):
        """Initialise a new ``Isbn`` object.

        Args:
            isbn (str): ISBN string

        """
        super(Isbn, self).__init__()
        self._isbn = isbn
        if len(isbn) in (9, 12):
            self.isbn = _isbn_cleanse(isbn, False)
        else:
            self.isbn = _isbn_cleanse(isbn)

    def __repr__(self):
        """Self-documenting string representation.


        Returns:
            ``str``: String to recreate ``Isbn`` object

        """
        return '%s(%r)' % (self.__class__.__name__, self.isbn)

    def __str__(self):
        """Pretty printed ISBN string.

        Returns:
            ``str``: Human readable string representation of ``Isbn`` object

        """
        return "ISBN %s" % self._isbn

    def __format__(self, format_spec):
        """Extended pretty printing for ISBN strings.

        Args:
            format_spec (str): Extended format to use

        Returns:
            ``str``: Human readable string representation of ``Isbn`` object

        Raises:
            ValueError: Unknown value for ``format_spec``

        """
        if not format_spec:  # default format calls set format_spec to ''
            return str(self)
        elif format_spec == 'url':
            return self.to_url()
        elif format_spec.startswith('url:'):
            parts = format_spec.split(':')[1:]
            site = parts[0]
            if len(parts) > 1:
                country = parts[1]
            else:
                country = 'us'
            return self.to_url(site, country)
        elif format_spec == 'urn':
            return self.to_urn()
        else:
            raise ValueError('Unknown format_spec %r' % format_spec)

    def calculate_checksum(self):
        """Calculate ISBN checksum.

        Returns:
            ``str``: ISBN checksum value

        """
        if len(self.isbn) in (9, 12):
            return calculate_checksum(self.isbn)
        else:
            return calculate_checksum(self.isbn[:-1])

    def convert(self, code='978'):
        """Convert ISBNs between ISBN-10 and ISBN-13.

        Args:
            code (str): ISBN-13 prefix code

        Returns:
            ``str``: Converted ISBN

        """
        return convert(self.isbn, code)

    def validate(self):
        """Validate an ISBN value.

        Returns:
            ``bool``: ``True`` if ISBN is valid

        """
        return validate(self.isbn)

    def to_url(self, site='amazon', country='us'):
        """Generate a link to an online book site.

        Args:
            site (str): Site to create link to
            country (str): Country specific version of ``site``

        Returns:
            ``str``: URL on ``site`` for book

        Raises:
            SiteError: Unknown site value
            CountryError: Unknown country value

        """
        try:
            try:
                url, tlds = URL_MAP[site]
            except ValueError:
                tlds = None
                url = URL_MAP[site]
        except KeyError:
            raise SiteError(site)
        inject = {'isbn': self.isbn}
        if tlds:
            if country not in tlds:
                raise CountryError(country)
            tld = tlds[country]
            if not tld:
                tld = country
            inject['tld'] = tld
        return url % inject

    def to_urn(self):
        """Generate a RFC 3187 URN.

        :rfc:`3187` is the commonly accepted way to use ISBNs as uniform
        resource names.

        Returns:
            ``str``: :rfc:`3187` compliant URN

        """
        return 'URN:ISBN:%s' % self._isbn


class Isbn10(Isbn):

    """Class for representing ISBN-10 objects.

    See also:
        ``Isbn``

    """

    def __init__(self, isbn):
        """Initialise a new ``Isbn10`` object.

        Args:
            isbn (str): ISBN-10 string

        """
        super(Isbn10, self).__init__(isbn)

    def calculate_checksum(self):
        """Calculate ISBN-10 checksum.

        Returns:
            ``str``: ISBN-10 checksum value

        """
        return calculate_checksum(self.isbn[:9])

    def convert(self, code='978'):
        """Convert ISBN-10 to ISBN-13.

        Args:
            code (str): ISBN-13 prefix code

        Returns:
            ``str``: ISBN-13 string

        """
        return convert(self.isbn, code)


class Sbn(Isbn10):

    """Class for representing SBN objects.

    See also:
        ``Isbn10``

    """

    def __init__(self, sbn):
        """Initialise a new ``Sbn`` object.

        Args:
            sbn (str): SBN string

        """
        isbn = '0' + sbn
        super(Sbn, self).__init__(isbn)

    def __repr__(self):
        """Self-documenting string representation.

        Returns:
            ``str``: String to recreate ``Sbn`` object

        """
        return '%s(%r)' % (self.__class__.__name__, self.isbn[1:])

    def calculate_checksum(self):
        """Calculate SBN checksum.

        Returns:
            ``str``: SBN checksum value

        """
        return calculate_checksum(self.isbn[:9])

    def convert(self, code='978'):
        """Convert SBN to ISBN-13.

        Args:
            code (str): ISBN-13 prefix code

        Returns:
            ``str``: ISBN-13 string

        """
        return super(Sbn, self).convert(code)


class Isbn13(Isbn):

    """Class for representing ISBN-13 objects.

    See also:
        ``Isbn``

    """

    def __init__(self, isbn):
        """Initialise a new ``Isbn13`` object.

        Args:
            isbn (str): ISBN-13 string

        """
        super(Isbn13, self).__init__(isbn)

    def calculate_checksum(self):
        """Calculate ISBN-13 checksum.

        Returns:
            ``str``: ISBN-13 checksum value

        """
        return calculate_checksum(self.isbn[:12])

    def convert(self, code=None):
        """Convert ISBN-13 to ISBN-10.

        Args:
            code: Ignored, only for compatibility with ``Isbn``

        Returns:
            ``str``: ISBN-10 string

        Raises:
            ValueError: When ISBN-13 isn't a Bookland "978" ISBN

        """
        return convert(self.isbn)


def _isbn_cleanse(isbn, checksum=True):
    """Check ISBN is a string, and passes basic sanity checks.

    Args:
        isbn (str): SBN, ISBN-10 or ISBN-13
        checksum (bool): ``True`` if ``isbn`` includes checksum character

    Returns:
        ``str``: ISBN with hyphenation removed, including when called with a SBN

    Raises:
        TypeError: ``isbn`` is not a ``str`` type
        IsbnError: Incorrect length for ``isbn``
        IsbnError: Incorrect SBN or ISBN formatting

    """
    if not isinstance(isbn, string_types):
        raise TypeError('ISBN must be a string, received %r' % isbn)

    if PY2 and isinstance(isbn, str):
        isbn = unicode(isbn)
        uni_input = False
    else:
        uni_input = True

    for dash in DASHES:
        isbn = isbn.replace(dash, unicode())

    if not isbn[:-1].isdigit():
        raise IsbnError('non-digit parts')
    if checksum:
        if len(isbn) == 9:
            isbn = "0" + isbn
        if len(isbn) == 10:
            if not (isbn[-1].isdigit() or isbn[-1] in "Xx"):
                raise IsbnError('non-digit or X checksum')
        elif len(isbn) == 13:
            if not isbn[-1].isdigit():
                raise IsbnError('non-digit checksum')
        else:
            raise IsbnError('ISBN must be either 10 or 13 characters long')
    else:
        if len(isbn) == 8:
            isbn = '0' + isbn
        if not isbn[-1].isdigit():
            raise IsbnError('non-digit parts')
        if not len(isbn) in (9, 12):
            raise IsbnError('ISBN must be either 9 or 12 characters long '
                            'without checksum')
    if PY2 and not uni_input:
        # Sadly, type ping-pong is required to maintain backwards compatibility
        # with previous pyisbn releases for Python 2 users.
        return str(isbn)
    else:
        return isbn


def calculate_checksum(isbn):
    """Calculate ISBN checksum.

    Args:
        isbn (str): SBN, ISBN-10 or ISBN-13

    Returns:
        ``str``: Checksum for given ISBN or SBN

    """
    isbn = [int(i) for i in _isbn_cleanse(isbn, checksum=False)]
    if len(isbn) == 9:
        products = [x * y for x, y in zip(isbn, range(1, 10))]
        check = sum(products) % 11
        if check == 10:
            check = 'X'
    else:
        # As soon as Python 2.4 support is dumped
        # [(isbn[i] if i % 2 == 0 else isbn[i] * 3) for i in range(12)]
        products = []
        for i in range(12):
            if i % 2 == 0:
                products.append(isbn[i])
            else:
                products.append(isbn[i] * 3)
        check = 10 - sum(products) % 10
        if check == 10:
            check = 0
    return str(check)


def convert(isbn, code='978'):
    """Convert ISBNs between ISBN-10 and ISBN-13.

    Note:
        No attempt to hyphenate converted ISBNs is made, because the
        specification requires that *any* hyphenation must be correct but
        allows ISBNs without hyphenation.

    Args:
        isbn (str): SBN, ISBN-10 or ISBN-13
        code (str): EAN Bookland code

    Returns:
        ``str``: Converted ISBN-10 or ISBN-13

    Raise:
        IsbnError: When ISBN-13 isn't convertible to an ISBN-10

    """
    isbn = _isbn_cleanse(isbn)
    if len(isbn) == 10:
        isbn = code + isbn[:-1]
        return isbn + calculate_checksum(isbn)
    else:
        if isbn.startswith('978'):
            return isbn[3:-1] + calculate_checksum(isbn[3:-1])
        else:
            raise IsbnError('Only ISBN-13s with 978 Bookland code can be '
                            'converted to ISBN-10.')


def validate(isbn):
    """Validate ISBNs.

    Warning:
        Publishers have been known to go to press with broken ISBNs, and
        therefore validation failures do not completely guarantee an ISBN is
        incorrectly entered.  It should however be noted that it is massively
        more likely *you* have entered an invalid ISBN than the published ISBN
        is incorrectly produced.  An example of this probability in the real
        world is that `Amazon <http://www.amazon.com/>`__ consider it so
        unlikely that they refuse to search for invalid published ISBNs.

    Args:
        isbn (str): SBN, ISBN-10 or ISBN-13

    Returns:
        ``bool``: ``True`` if ISBN is valid

    """
    isbn = _isbn_cleanse(isbn)
    return isbn[-1].upper() == calculate_checksum(isbn[:-1])



#
# coding=utf-8
"""test_data - ISBNs for use in tests"""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#: Sample book data for use in tests
TEST_BOOKS = {
    '20, 000 Twenty Thousand Leagues Under the Sea': '0-14-062118-0',
    '3D Math Primer for Graphics and Game Development': '1-55622-911-9',
    'A Course in Combinatorics': '0-521-00601-5',
    'A Course in Modern Mathematical Physics: Groups, Hilbert Space and Differential Geometry': '0-521-82960-7',
    'A First Course in Coding Theory (Oxford Applied Mathematics & Computing Science S.)': '0-19-853803-0',
    'A First Course in General Relativity': '0-521-27703-5',
    'A First Course in String Theory': '0-521-83143-1',
    'A New Kind of Science': '1-57955-008-8',
    'ADA in Distributed Real Time Systems': '0-07-046544-4',
    'Ada 2005 Reference Manual': '3-540-69335-1',
    'Advanced Engineering Mathematics': '1-4039-0312-3',
    'Advanced Signal Processing Algorithms, Architectures, and Implementation IX (SPIE)': '0-8194-3293-8',
    'Aerodynamics for Engineering Students': '0-7506-5111-3',
    'Aerospace Avionics Systems: A Modern Synthesis': '978-0-12-646890-8',
    'An Introduction to Fire Dynamics': '0-471-97291-6',
    'An Introduction to Modern Astrophysics (International Edition)': '0-321-21030-1',
    'An Introduction to Radio Astronomy': '0-521-00517-5',
    'Antenna Arraying Techniques in the Deep Space Network': '0-471-46799-5',
    'Applied Combinatorics': '0-471-43809-X',
    'Applied Cryptography: Protocols, Algorithms and Source Code in C': '0-471-11709-9',
    'Applied Satellite Navigation Using GPS, GALILEO, and Augmentation Systems': '1-58053-814-2',
    'Archimedes Effect': '0-14-101143-2',
    'Artificial Intelligence: A Modern Approach (International Edition)': '0-13-080302-2',
    'Artificial Intelligence: Structures and Strategies for Complex Problem Solving': '0-321-26318-9',
    'Astronomical Algorithms': '0-943396-61-1',
    'At the Bench: A Laboratory Navigator': '0-87969-708-3',
    'Automatic Target Recognition IX (SPIE)': '0-8194-3192-3',
    'Autonomous Software-defined Radio Receivers for Deep Space Applications': '0-470-08212-7',
    'Avionics Troubleshooting and Repair': '978-0-07-136495-9',
    'Beyond Fear: Thinking Sensibly About Security in an Uncertain World': '0-387-02620-7',
    'Brunel: The Man Who Built the World': '0-297-84408-3',
    'Burning Chrome': '978-0-00-648043-3',
    'CFD: A Practical Approach': '0-7506-8563-8',
    'Carbon Fibers and Their Composites': '978-0-8247-0983-9',
    'Carrie': '978-0-385-08695-0',
    'Causality: Models, Reasoning, and Inference': '0-521-77362-8',
    'Chaos: An Introduction to Dynamical Systems (Textbooks in Mathematical Sciences S.)': '0-387-94677-2',
    'Chaos and Complexity in Astrophysics': '0-521-85534-9',
    'Civil Engineering Formulas': '0-07-135612-6',
    'Classical Mechanics: An Undergraduate Text': '0-521-53409-7',
    'Clear and Present Danger': '0-00-617730-1',
    "Colossus: The Secrets of Bletchley Park's Code-breaking Computers": '0-19-284055-X',
    'Combinatorial Auctions': '0-262-03342-9',
    'Combinatorics: Topics, Techniques, Algorithms': '0-521-45761-0',
    'Computational Fluid Dynamics': '0-521-76969-8',
    'Count Zero': '0-575-03696-6',
    'Creating More Effective Graphs': '0-471-27402-X',
    'Crime Scene Investigation: Methods and Procedures': '0-335-21490-8',
    'Crime Scene to Court: The Essentials of Forensic Science': '0-85404-656-9',
    'Criminalistics: An Introduction to Forensic Science': '0-13-122889-7',
    'Cryptography Theory and Practice': '1-58488-508-4',
    'Cryptonomicon': '978-0-09-941067-6',
    'Cybernetics: Or Control and Communication in the Animal and the Machine': '0-262-73009-X',
    'Cyberpunk and Cyberculture: Science Fiction and the Work of William Gibson': '0-485-00607-3',
    'Data Analysis: A Bayesian Tutorial': '0-19-856832-0',
    'Dead Reckoning: Calculating Without Instruments': '0-88415-087-9',
    'Deep Down Things: The Breathtaking Beauty of Particle Physics': '0-8018-7971-X',
    'Democratizing Innovation': '0-262-00274-4',
    'Dependence Logic: A New Approach to Independence Friendly Logic': '0-521-70015-9',
    'Design patterns : elements of reusable object-oriented software': '0-201-63361-2',
    'Dictionary of British Sign Language': '0-571-14346-6',
    'Discovering Genomics, Proteomics and Bioinformatics': '0-8053-8219-4',
    'Discovering Statistics Using SPSS': '0-7619-4452-4',
    'Divine Comedy': '978-0-631-12926-4',
    'Don Quixote': '978-1-85326-036-0',
    'Dracula': '978-0-19-956409-5',
    "Einstein's Miraculous Year: Five Papers That Changed the Face of Physics": '0-691-12228-8',
    'Electronic Navigation Systems': '978-0-7506-5138-7',
    'Elements of Photogrammetry with Applications in GIS': '0-07-292454-3',
    'Engineering Mathematics 5th ed': '0-333-91939-4',
    'Essential Computational Fluid Dynamics': '0-470-42329-3',
    'Estimation with Applications to Tracking and Navigation': '0-471-41655-X',
    'Evolution of Networks: From Biological Nets to the Internet and WWW': '0-19-851590-1',
    'Factory Physics': '0-07-116378-6',
    'Forensic Science': '0-13-043251-2',
    'Friendly Introduction to Graph Theory': '0-13-066949-0',
    'From the Earth to the Moon': '0-553-21420-9',
    'Fundamentals of Biomechanics': '0-306-47474-3',
    'Fundamentals of Heat and Mass Transfer': '0-471-38650-2',
    'Fundamentals of the Physical Environment': '0-415-23294-5',
    'Gauge Theories in Particle Physics: From Relativistic Quantum Mechanics to QED: v. 1': '0-7503-0864-8',
    'Generalized Latent Variable Modeling: Multilevel, Longitudinal, & Structural Equation Models': '1-58488-000-7',
    'Generating Families in the Restricted Three-Body Problem (Lecture Notes in Physics S.)': '3-540-63802-4',
    'Generating Families in the Restricted Three-body Problem: Quantitative Study of Bifurcations: Pt. 2 (Lecture Notes in Physics)': '3-540-41733-8',
    'Genetic Algorithms in Search, Optimization and Machine Learning': '0-201-15767-5',
    'Getting Things Done: The Art of Stress-Free Productivity': '0-670-88906-7',
    'Global Navigation Satellite System (GNSS) Receivers for Weak Signals': '1-59693-052-7',
    'Global Positioning System: Theory and Applications': '978-1-56347-107-0',
    'Gonzo Gizmos: Projects and Devices to Channel Your Inner Geek': '1-55652-520-6',
    'Gravity from the Ground Up: An Introductory Guide to Gravity and General Relativity': '0-521-45506-5',
    'Ground Penetrating Radar': '0-86341-360-9',
    'Guide to Biometrics (Springer Professional Computing S.)': '0-387-40089-3',
    'Guide to Elliptic Curve Cryptography (Springer Professional Computing)': '0-387-95273-X',
    'Hackers and Painters: Essays on the Art of Programming': '0-596-00662-4',
    'Hacking GPS': '0-7645-8424-3',
    'Handbook of Applied Cryptography': '0-8493-8523-7',
    'Handbook of Fingerprint Recognition (Springer Professional Computing S.)': '0-387-95431-7',
    'Heat and Mass Transfer': '1-904798-47-0',
    'Heat and Thermodynamics': '0-07-114816-7',
    'High Integrity Software': '0-321-13616-0',
    'How to Research': '0-335-20903-3',
    'How to Use a Computerized Telescope: Practical Amateur Astronomy Volume 1: v. 1 (Practical Amateur Astronomy)': '0-521-00790-9',
    'IA-64 Linux Kernel: Design and Implementation': '0-13-061014-3',
    'Icon Steve Jobs: The Greatest Second Act in the History of Business': '0-471-72083-6',
    'In Search of Dark Matter: The Search for Dark Matter in the Universe (Springer-Praxis Books S.)': '0-387-27616-5',
    'Information and Coding Theory (Springer Undergraduate Mathematics S.)': '1-85233-622-6',
    'Introduction to Algorithms': '0-262-53196-8',
    'Introduction to Elementary Particles': '0-471-60386-4',
    'Introduction to Mechanics': '0-07-085423-8',
    'Introduction to Quantum Computation and Information': '981-02-4410-X',
    'Introduction to Space Physics (Cambridge Atmospheric & Space Science S.)': '0-521-45714-9',
    'Introduction to Statistical Physics': '0-7484-0942-4',
    'Isambard Kingdom Brunel': '0-14-011752-0',
    'Joel on Software: And on Diverse and Occasionally Related Matters That Will Prove of Interest to Software Developers, Designers, and Managers, and to Those Who, Whether by Good Fortune or Ill-Luck, Work with Them in Some Capacity': '1-59059-389-8',
    'Journey to the Centre of the Earth': '0-14-062139-3',
    'Knots and Surfaces: A Guide to Discovering Mathematics': '0-8218-0451-0',
    'Knowledge Representation: Logical, Philosophical and Computational Foundations': '0-534-94965-7',
    'LDAP in the Solaris Operating Environment: Deploying Secure Directory Services': '0-13-145693-8',
    "Let's Sign Dictionary: Everyday BSL for Learners": '0-9542384-3-5',
    'Linux Kernel Development': '0-672-32720-1',
    'Local Search in Combinatorial Optimization': '0-691-11522-2',
    "Lunar and Planetary Webcam User's Guide (Patrick Moore's Practical Astronomy S.)": '1-84628-197-0',
    'Malicious Cryptography: Exposing Cryptovirology': '0-7645-4975-8',
    'Manual of Engineering Drawing': '0-7506-5120-2',
    'Mapping Hacks: Tips & Tools for Electronic Cartography': '978-0-596-00703-4',
    'Masters of Doom: How Two Guys Created an Empire and Transformed Pop Culture': '0-7499-2489-6',
    'Mathematical Handbook for Scientists and Engineers: Definitions, Theorems, and Formulas for Reference and Review': '0-486-41147-8',
    'Mechanics (Course of Theoretical Physics)': '0-7506-2896-0',
    'MicroC/OS II: The Real Time Kernel': '1-57820-103-9',
    'Microfluidics for Biotechnology': '1-58053-961-0',
    "Microlight Pilot's Handbook": '978-1-84037-286-1',
    'Microsoft Application Architecture Guide v2': '978-0-7356-2710-9',
    'Mobile Satellite Communications Handbook': '978-0-471-29778-9',
    'Modern Supersymmetry: Dynamics and Duality (International Series of Monographs on Physics)': '0-19-856763-4',
    'Neuromancer': '0-441-56956-0',
    'Mona Lisa Overdrive': '978-0-00-648044-0',
    'Nonlinear Control Systems (Communications & Control Engineering S.)': '3-540-19916-0',
    'Nuclear and Particle Physics': '0-582-45088-8',
    'Numerical Heat Transfer and Fluid Flow': '0-89116-522-3',
    'Numerical Recipes in C++: The Art of Scientific Computing': '0-521-75033-4',
    'Paradigms of Artificial Intelligence Programming: Case Studies in Common LISP': '1-55860-191-0',
    'Particle Astrophysics (Oxford Master Series in Physics)': '0-19-850952-9',
    'Particle Physics (Manchester Physics S.)': '0-471-97285-1',
    'Pattern Recognition': '978-0-241-95353-2',
    'Permutation City': '1-85798-218-5',
    'Physical Design Essentials': '0-387-36642-3',
    'Power and Invention: Situating Science (Theory Out of Bounds S.)': '0-8166-2517-4',
    'Practical Statistics for Astronomers': '0-521-45616-9',
    'Predicting Motion: The Physical World (The Physical World)': '0-7503-0716-1',
    'Probabilistic Robotics (Intelligent Robotics & Autonomous Agents S.)': '0-262-20162-3',
    'Probability Theory: The Logic of Science: Principles and Elementary Applications Vol 1': '0-521-59271-2',
    'Programming and Customizing the Multicore Propeller Microcontroller': '978-0-07-166450-9',
    'Programming in Haskell': '978-0-521-87172-3',
    'Quantum Groups: A Path to Current Algebra': '0-521-69524-4',
    'Radiation Effects on Embedded System': '978-1-4020-5645-1',
    'Radiometric Tracking Techniques for Deep-Space Navigation': '0-471-44534-7',
    'Rainbow Six': '0-14-027405-7',
    'Ramsey Theory on the Integers (Student Mathematical Library)': '0-8218-3199-2',
    'Random Graph Dynamics': '0-521-86656-1',
    'Reliable Embedded Systems': '0-321-25291-8',
    'Reliable Software Technology': '3-540-26286-5',
    'Renormalization Methods: A Guide for Beginners ': '0-19-850694-5',
    'Resilience Engineering: Concepts and Precepts': '0-7546-4904-0',
    'Revolutionaries of the Cosmos: The Astro-Physicists': '0-19-857099-6',
    "Schaum's Outline of Feedback and Control Systems (Schaum S.)": '0-07-017052-5',
    'Secrets and Lies: Digital Security in a Networked World': '0-471-45380-3',
    'Security Engineering: A Guide to Building Dependable Distributed Systems': '0-471-38922-6',
    'Sensor Modelling and Data Processing for Autonomous Navigation': '981-02-3496-1',
    'Show Me the Numbers: Designing Tables and Graphs to Enlighten': '0-9706019-9-9',
    'Signal Integrity Effects in Custom IC and ASIC Designs': '0-471-15042-8',
    'Small Worlds: The Dynamics of Networks Between Order and Randomness (Princeton Studies in Complexity)': '0-691-11704-7',
    'Snow Crash': '978-0-241-95318-1',
    'Softwar: An Intimate Portrait of Larry Ellison and Oracle': '0-7432-2505-8',
    'Software as Capital: Economic Perspective on Software Engineering': '0-8186-7779-1',
    'Solaris: Systems Programming': '0-201-75039-2',
    'Spook Country': '978-0-241-95354-9',
    'Statistical Analysis of Circular Data': '0-521-56890-0',
    'Statistical Mechanics': '9971-966-07-7',
    'Statistical Mechanics: A Survival Guide': '0-19-850816-6',
    'Statistical Mechanics: Entropy, Order Parameters and Complexity': '0-19-856677-8',
    'Statistical Mechanics of Lattice Systems: Exact, Series and Renormalization Group Methods: v. 2 (Texts & Monographs in Physics)': '3-540-64436-9',
    'Statistics: An Introduction Using R': '0-470-02298-1',
    'Storming the Reality Studio: Casebook of Cyberpunk and Postmodern Science Fiction': '0-8223-1168-2',
    'Strange Attractors: Chaos, Complexity and the Art of Family Therapy (Wiley Series in Couples & Family Dynamics & Treatment)': '0-471-07951-0',
    'Strapdown Inertial Navigation Technology': '0-86341-358-7',
    'Swimsuit': '978-1-84605-262-0',
    'Synthesis of Arithmetic Circuits': '0-471-68783-9',
    'Techniques of Crime Scene Investigation': '0-8493-1691-X',
    'The ASIC Handbook': '0-13-091558-0',
    'The Art of Intrusion: The Real Stories Behind the Exploits of Hackers, Intruders and Deceivers': '0-7645-6959-7',
    'The Art of Project Management': '0-596-00786-8',
    'The Castle': '978-0-19-923828-6',
    'The Computational Beauty of Nature: Computer Explorations of Fractals, Chaos, Complex Systems and Adaptation (Bradford Book S.)': '0-262-56127-1',
    'The Concepts and Practice of Mathematical Finance (Mathematic, Finance & Risk S.)': '0-521-82355-2',
    'The Damage Tolerance of Warp Knit Fabric Polymer Composites': '978-3-639-23478-7',
    'The Difference Engine': '0-575-04762-3',
    'The Economics of the European Patent System': '0-19-921698-3',
    'The Egyptian Calendar: A Work for Eternity': '1-902699-05-X',
    'The First Men in the Moon': '0-460-87304-0',
    'The Girl with the Dragon Tattoo': '978-1-84724-253-2',
    'The HP Way: How Bill Hewlett and I Built Our Company': '0-06-084579-1',
    'The Hunt for Red October': '0-00-617276-8',
    'The Idiot': '978-0-14-044792-7',
    'The International Standard Book Number System': '3-88053-101-3',
    'The Invisible Man': '0-460-87628-7',
    'The Marine Electrical and Electronics Bible': '978-1-57409-242-4',
    'The Meaning of It All (Allen Lane History S.)': '0-14-027635-1',
    'The Mythical Man Month and Other Essays on Software Engineering': '0-201-83595-9',
    'The Principia: Mathematical Principles of Natural Philosophy': '0-520-08817-4',
    'The Statistical Mechanics of Financial Markets (Texts & Monographs in Physics)': '3-540-00978-7',
    'The Thermodynamics Problem Solver': '0-87891-555-9',
    'The Time Machine': '0-460-87735-6',
    'The Tipping Point': '0-316-31696-2',
    'The Trial': '978-0-14-118290-2',
    'The Trouble with Physics': '0-7139-9799-0',
    'The Universal Computer: The Road from Leibniz to Turing': '0-393-04785-7',
    'The Visual Display of Quantitative Information': '0-9613921-4-2',
    'The War of the Worlds': '0-460-87303-2',
    'Theory of Financial Risk and Derivative Pricing: From Statistical Physics to Risk Management': '0-521-81916-4',
    'Thermal Physics': '0-521-65838-1',
    'Thermodynamics of Natural Systems': '0-521-84772-9',
    'Thinking Forth': '0-9764587-0-5',
    'Time Synchronization Application in Wireless Sensor Network': '978-3-639-00957-6',
    "Time's Alteration: Calendar Reform in Early Modern England": '1-85728-622-7',
    'Turbo Codes: Principles and Applications (Kluwer International Series in Engineering & Computer Science)': '0-7923-7868-7',
    'Turbulence and Structures: Chaos, Fluctuations and Helical Self-organization in Nature and Laboratory (A Volume in the INTERNATIONAL GEOPHYSICS Series)': '0-12-125740-1',
    'Under the Dome': '978-0-340-99258-6',
    'Understanding Energy: Energy, Entropy and Thermodynamics for Everyman': '981-02-0679-8',
    'Understanding the Linux Kernel': '0-596-00565-2',
    'User Interface Design for Programmers': '1-893115-94-1',
    'VLSI: Memory, Microprocessor and ASIC': '0-8493-1737-1',
    'Vingt mille lieues sous les mers': '978-2-01-322778-0',
    'Virtual Light': '978-0-670-85833-0',
    'Visual Complex Analysis': '0-19-853446-9',
    'WIG Craft and Ekranoplan: Ground Effect Craft Technology': '978-1-4419-0041-8',
    'What Are the Chances?: Voodoo Deaths, Office Gossip and Other Adventures in Probability': '0-8018-6941-2',
    'Without Remorse': '0-00-647641-4',
    'Wizard: Life and Times of Nikola Tesla': '0-8065-1960-6',
    'Zero History': '978-0-670-91952-9'
}



#! /usr/bin/env python
# coding=utf-8
"""setup.py - Setuptools tasks and config for pyisbn"""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

try:
    from email.utils import parseaddr
except ImportError:  # Python2.4
    from email.Utils import parseaddr  # NOQA

from setuptools import setup

import pyisbn


author, author_email = parseaddr(pyisbn.__author__)

paras = pyisbn.__doc__.split('\n\n')
long_description = "\n\n".join([paras[1], paras[3]])

setup(
    name='pyisbn',
    version=pyisbn.__version__,
    description=pyisbn.__doc__.splitlines()[0][:-1],
    long_description=long_description,
    author=author,
    author_email=author_email,
    maintainer=author,
    maintainer_email=author_email,
    url="https://github.com/JNRowe/pyisbn/",
    packages=['pyisbn', ],
    license=pyisbn.__license__,
    keywords='ISBN ISBN-10 ISBN-13 SBN',
    classifiers=[
        'Development Status :: 6 - Mature',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.4',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.0',
        'Programming Language :: Python :: 3.1',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Other/Nonlisted Topic',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing',
        'Topic :: Text Processing :: Indexing',
    ],
)



#
# coding=utf-8
"""test_regressions - Test for regressions"""
# Copyright © 2007-2016  James Rowe <jnrowe@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import unicodedata

from expecter import expect
from nose2.tools import params

from pyisbn import _isbn_cleanse


# The lookup hoop jumping here is make it easier to generate native Unicode
# types on the various supported Python versions.
@params(
    '978-1-84724-253-2',
    unicodedata.lookup('EN DASH').join(['978', '1', '84724', '253', '2']),
    unicodedata.lookup('HORIZONTAL BAR').join(['978', '0199564095']),
)
def test_issue_7_unistr(isbn):
    expect(_isbn_cleanse(isbn)) == "".join(filter(lambda s: s.isdigit(), isbn))
